package djmil.cordacheckers.cordaclient;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;
import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import djmil.cordacheckers.cordaclient.dao.GameView;
import djmil.cordacheckers.cordaclient.dao.GameView.Status;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.RspFailureException;
import djmil.cordacheckers.cordaclient.dao.Stone;
import djmil.cordacheckers.user.HoldingIdentityResolver;

@SpringBootTest
public class GameProposalTests {
    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    final String issuer  = "alice";
    final String acquier = "bobik";
    final Stone.Color acquierColor = Stone.Color.WHITE;

    @Test
    void testCreate() throws RspFailureException {
        final var hiIssuer  = holdingIdentityResolver.getByUsername(issuer);
        final var hiAcquier = holdingIdentityResolver.getByUsername(acquier);
        final String message = "GameProposal CREATE test";

        final GameView issuerGameView = cordaClient.gameProposalCreate(
            hiIssuer, hiAcquier, acquierColor, message);

        /*
         * Issuers perspective on a newly created GameProposal
         */
        assertThat(issuerGameView.status()).isEqualByComparingTo(Status.GAME_PROPOSAL_WAIT_FOR_OPPONENT);
        assertThat(issuerGameView.myColor()).isEqualByComparingTo(acquierColor.opposite());
        assertThat(issuerGameView.opponentName()).isEqualToIgnoringCase(acquier);
        assertThat(issuerGameView.board()).containsAllEntriesOf(GameView.defaultGameBoard);
        assertThat(issuerGameView.previousMove()).isNull();
        assertThat(issuerGameView.moveNumber() == 0);
        assertThat(issuerGameView.message()).isEqualToIgnoringCase(message);
        assertThat(issuerGameView.uuid()).isNotNull();
      
        /*
         * Acquier shall be able to find newly created GameProposal
         */
        GameView acquierGameView = assertDoesNotThrow( () -> {
            return cordaClient.gameStateList(hiAcquier)
                .stream()
                .filter(gameView -> gameView.uuid().compareTo(issuerGameView.uuid()) == 0)
                .reduce((a, b) -> {throw new IllegalStateException("Duplicate");})
                .get();
        });

        assertThat(acquierGameView.status()).isEqualByComparingTo(Status.GAME_PROPOSAL_WAIT_FOR_YOU);
        assertThat(acquierGameView.myColor()).isEqualByComparingTo(acquierColor);
        assertThat(acquierGameView.opponentName()).isEqualToIgnoringCase(issuer);
        assertThat(acquierGameView.board()).containsAllEntriesOf(GameView.defaultGameBoard);
        assertThat(acquierGameView.previousMove()).isNull();
        assertThat(acquierGameView.moveNumber() == 0);
        assertThat(acquierGameView.message()).isEqualToIgnoringCase(message);
        assertThat(acquierGameView.uuid()).isEqualByComparingTo(issuerGameView.uuid());
    }

    @Test
    void testReject() throws RspFailureException {
        final var hiIssuer  = holdingIdentityResolver.getByUsername(issuer);
        final var hiAcquier = holdingIdentityResolver.getByUsername(acquier);

        final GameView game = cordaClient.gameProposalCreate(
            hiIssuer, hiAcquier, acquierColor, "GameProposal REJECT test");

        assertThatThrownBy(() -> {  // Issuer can not reject
            cordaClient.gameProposalReject(
                hiIssuer, game.uuid());
        });

        final GameView pendingGameView = cordaClient.gameStateGet(
            hiAcquier, game.uuid());
        assertThat(pendingGameView.status()).isEqualByComparingTo(Status.GAME_PROPOSAL_WAIT_FOR_YOU);
        assertThat(pendingGameView.uuid()).isEqualByComparingTo(game.uuid());

        final GameView rejectedGameView = cordaClient.gameProposalReject(
            hiAcquier, game.uuid());
        assertThat(rejectedGameView.status()).isEqualByComparingTo(Status.GAME_PROPOSAL_REJECTED);
        assertThat(rejectedGameView.uuid()).isEqualByComparingTo(game.uuid());

        assertThatThrownBy(() -> {  // Issuer has no pending GameProposal
            cordaClient.gameStateGet(
                hiIssuer, game.uuid());
        });
    }

    @Test
    void testCancel() throws RspFailureException {
        final var hiIssuer  = holdingIdentityResolver.getByUsername(issuer);
        final var hiAcquier = holdingIdentityResolver.getByUsername(acquier);

        final GameView game = cordaClient.gameProposalCreate(
            hiIssuer, hiAcquier, acquierColor, "GameProposal CANCEL test");

        assertThatThrownBy(() -> {  // Acquier can not cancel
            cordaClient.gameProposalCancel(
                hiAcquier, game.uuid());
        });

        final GameView pendingGameView = cordaClient.gameStateGet(
            hiIssuer, game.uuid());
        assertThat(pendingGameView.status()).isEqualByComparingTo(Status.GAME_PROPOSAL_WAIT_FOR_OPPONENT);
        assertThat(pendingGameView.uuid()).isEqualByComparingTo(game.uuid());

        final GameView canceledGameView = cordaClient.gameProposalCancel(
            hiIssuer, game.uuid());
        assertThat(canceledGameView.status()).isEqualByComparingTo(Status.GAME_PROPOSAL_CANCELED);
        assertThat(canceledGameView.uuid()).isEqualByComparingTo(game.uuid());

        assertThatThrownBy(() -> {  // Acquier has no pending GameProposal
            cordaClient.gameStateGet(
            hiAcquier, game.uuid());
        });
    }

    @Test
    void testAccept() throws RspFailureException {
        final var hiIssuer  = holdingIdentityResolver.getByUsername(issuer);
        final var hiAcquier = holdingIdentityResolver.getByUsername(acquier);

        final GameView game = cordaClient.gameProposalCreate(
            hiIssuer, hiAcquier, acquierColor, "GameProposal ACCEPT test");

        assertThatThrownBy(() -> { // Issuer can not accept
            cordaClient.gameProposalAccept(hiIssuer, game.uuid());
        });

        final GameView acceptedGameAcquierView = cordaClient.gameProposalAccept(
            hiAcquier, game.uuid());

        assertThat(acceptedGameAcquierView.myColor()).isEqualByComparingTo(acquierColor);
        assertThat(acceptedGameAcquierView.opponentName()).isEqualToIgnoringCase(issuer);
        assertThat(acceptedGameAcquierView.board()).containsAllEntriesOf(GameView.defaultGameBoard);
        assertThat(acceptedGameAcquierView.previousMove()).isNull();
        assertThat(acceptedGameAcquierView.moveNumber() == 0);
        assertThat(acceptedGameAcquierView.status()).isEqualByComparingTo(Status.GAME_BOARD_WAIT_FOR_YOU);
        assertThat(acceptedGameAcquierView.uuid()).isEqualByComparingTo(game.uuid());

        assertThatThrownBy(() -> {   // same GameProposal can not be accepted twice 
            cordaClient.gameProposalAccept(
                hiIssuer, game.uuid());
        });

        final GameView acceptedGameIssuerView = cordaClient.gameStateGet(
            hiIssuer, game.uuid());

        assertThat(acceptedGameIssuerView.myColor()).isEqualByComparingTo(acquierColor.opposite());
        assertThat(acceptedGameIssuerView.opponentName()).isEqualToIgnoringCase(acquier);
        assertThat(acceptedGameIssuerView.board()).containsAllEntriesOf(GameView.defaultGameBoard);
        assertThat(acceptedGameIssuerView.previousMove()).isNull();
        assertThat(acceptedGameIssuerView.moveNumber() == 0);
        assertThat(acceptedGameIssuerView.status()).isEqualByComparingTo(Status.GAME_BOARD_WAIT_FOR_OPPONENT);
        assertThat(acceptedGameIssuerView.uuid()).isEqualByComparingTo(game.uuid());
    }

}
