package djmil.cordacheckers.gameproposal;

import static java.util.Objects.requireNonNull;

import java.time.Duration;
import java.time.Instant;
import java.util.Arrays;
import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.VNode;
import djmil.cordacheckers.checkers.Stone;
import djmil.cordacheckers.contracts.GameCommand;
import djmil.cordacheckers.gamestate.CommitTrx;
import djmil.cordacheckers.gamestate.FlowResponce;
import djmil.cordacheckers.gamestate.View;
import djmil.cordacheckers.gamestate.ViewBuilder;
import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.common.NotaryLookup;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;
import net.corda.v5.membership.MemberInfo;
import net.corda.v5.membership.NotaryInfo;

public class CreateFlow implements ClientStartableFlow{

     private final static Logger log = LoggerFactory.getLogger(CreateFlow.class);

     @CordaInject
     public JsonMarshallingService jsonMarshallingService;

     @CordaInject
     public MemberLookup memberLookup;

     @CordaInject
     NotaryLookup notaryLookup;

     @CordaInject
     public UtxoLedgerService utxoLedgerService;

     @CordaInject
     public FlowEngine flowEngine;

     @Suspendable
     @Override
     public String call(ClientRequestBody requestBody) {
          SecureHash utxoTrxId = null;

          try {
               final GameCommand command = new GameCommand(GameCommand.Action.GAME_PROPOSAL_CREATE);
               final MemberInfo custodianInfo = findCustodian();

               final GameProposalState gameProposal = buildGameProposalStateFrom(requestBody, custodianInfo);
               
               final UtxoSignedTransaction gameProposalCreateTrx = utxoLedgerService.createTransactionBuilder()
                    .addCommand(command)
                    .addOutputState(gameProposal)
                    .addSignatories(gameProposal.getParticipants())
                    .setNotary(findNotary().getName())
                    .setTimeWindowUntil(Instant.now().plusMillis(Duration.ofDays(1).toMillis()))
                    .toSignedTransaction();

               utxoTrxId = this.flowEngine
                    .subFlow(new CommitTrx(gameProposalCreateTrx, gameProposal.getParticipants()));

               final View gameView = this.flowEngine
                    .subFlow(new ViewBuilder(utxoTrxId));

               return new FlowResponce(gameView, utxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          } 
          catch (Exception e) {
               log.warn(requestBody + " [ERROR] " +e.toString());
               return new FlowResponce(e, utxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          }
     }

     @Suspendable
     GameProposalState buildGameProposalStateFrom(ClientRequestBody requestBody, MemberInfo custodianInfo) {
          final CreateFlowArgs args = requestBody.getRequestBodyAs(jsonMarshallingService, CreateFlowArgs.class);

          final MemberInfo myInfo = memberLookup.myInfo();
          final MemberInfo opponentInfo = requireNonNull(
               memberLookup.lookup(MemberX500Name.parse(args.opponentName)),
               "MemberLookup can't find opponentName specified in flow arguments: " + args.opponentName
          );

          final MemberInfo whitePlayerInfo = args.opponentColor == Stone.Color.WHITE ? opponentInfo : myInfo;
          final MemberInfo blackPlayerInfo = args.opponentColor == Stone.Color.BLACK ? opponentInfo : myInfo;

          return new GameProposalState(
               whitePlayerInfo.getName(),
               blackPlayerInfo.getName(),
               myInfo.getName(), // <<--- GameProposal issuer
               UUID.randomUUID(),
               args.board,
               args.message,
               Arrays.asList(
                    myInfo.getLedgerKeys().get(0), 
                    opponentInfo.getLedgerKeys().get(0),
                    custodianInfo.getLedgerKeys().get(0)
                    )
          );
     }

     @Suspendable
     public NotaryInfo findNotary() {
         return notaryLookup.getNotaryServices()
             .stream()
             .filter(notary -> VNode.isCordaCherckersNotary(notary))
             .reduce((a,b) -> {throw new IllegalStateException("Multiple Notary VNodes");})
             .orElseThrow( () -> new IllegalStateException("No Notary VNode found"));
     }

     @Suspendable
     MemberInfo findCustodian() {
         return memberLookup.lookup()
             .stream()
             .filter(member -> VNode.isCordaCherckersCustodian(member) )
             .reduce((a,b) -> {throw new IllegalStateException("Multiple Custodian VNodes");})
             .orElseThrow( () -> new IllegalStateException("No Custodian VNode found"));
     }
}
