import './GameSelector.css';
import React, { useContext } from 'react';
import { GamesStateContext, GamesGuideContext } from '../../context/games';

import { Board, Color, Player } from '../../components/Checkers';
import Loading from '../../components/Loading';
import Counter from '../../components/Counter';

export function GameProposalSelector({ onSelect }) {
    const games = useContext(GamesStateContext);
    const guide = useContext(GamesGuideContext);

    const isSelected = (uuid) => uuid === guide.selectedUUID.proposal;

    const onClick = (uuid) => {
        if (isSelected(uuid))
            onSelect(null); // deselect previously selected game
        else
            onSelect(uuid);
    }

    const yoursList = games.filter(game => game.status === 'GAME_PROPOSAL_WAIT_FOR_YOU')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const opponentsList = games.filter(game => game.status === 'GAME_PROPOSAL_WAIT_FOR_OPPONENT')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    if (yoursList.length === 0 && opponentsList.length === 0) {
        if (guide.isPolling)
            return <Loading />
        else
            return <>There are no pending GameProposals..</>
    }

    return (
        <div className='GameSelector'>
            {yoursList}
            {opponentsList.length > 0 && <Separator counter={opponentsList.length}>waiting for opponent</Separator>}
            {opponentsList}
        </div>
    )
}

export function ActiveGameSelector({ onSelect }) {
    const games = useContext(GamesStateContext);
    const guide = useContext(GamesGuideContext);

    const isSelected = (uuid) => uuid === guide.selectedUUID.active;

    const onClick = (uuid) => {
        if (isSelected(uuid))
            onSelect(null); // deselect previously selected game
        else
            onSelect(uuid);
    }

    const yoursList = games.filter(game => (game.status === 'GAME_BOARD_WAIT_FOR_YOU' || game.status === 'DRAW_REQUEST_WAIT_FOR_YOU'))
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const opponentsList = games.filter(game => (game.status === 'GAME_BOARD_WAIT_FOR_OPPONENT' || game.status === 'DRAW_REQUEST_WAIT_FOR_OPPONENT'))
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    if (yoursList.length === 0 && opponentsList.length === 0) {
        if (guide.isPolling)
            return <Loading />
        else
            return <>There are no pending Games..</>
    }

    return (
        <div className='GameSelector'>
            {yoursList}
            {opponentsList.length > 0 && <Separator counter={opponentsList.length}>waiting for opponent</Separator>}
            {opponentsList}
        </div>
    )
}

export function GameArchiveSelector({ onSelect }) {
    const games = useContext(GamesStateContext);
    const guide = useContext(GamesGuideContext);

    const isSelected = (uuid) => uuid === guide.selectedUUID.archive;

    const onClick = (uuid) => {
        if (isSelected(uuid))
            onSelect(null); // deselect previously selected game
        else
            onSelect(uuid);
    }

    const rejectedList = games.filter(game => game.status === 'GAME_PROPOSAL_REJECTED')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const canceledList = games.filter(game => game.status === 'GAME_PROPOSAL_CANCELED')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const victoryList = games.filter(game => game.status === 'GAME_RESULT_YOU_WON')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const defeatList = games.filter(game => game.status === 'GAME_RESULT_YOU_LOOSE')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    const drawList = games.filter(game => game.status === 'GAME_RESULT_DRAW')
        .map(game => <Selectable game={game} key={game.uuid} selected={isSelected(game.uuid)} onClick={onClick} />)

    if (rejectedList.length === 0 && canceledList.length === 0 && victoryList.length === 0 && defeatList.length === 0 && drawList.length === 0) {
        if (guide.isPolling)
            return <Loading />
        else
            return <>Finished Games will be shown here..</>
    }

    return (
        <div className='GameSelector'>
            {rejectedList.length > 0 && <Separator counter={rejectedList.length}>rejected proposals</Separator>}
            {rejectedList}
            {canceledList.length > 0 && <Separator counter={canceledList.length}>canceled proposals</Separator>}
            {canceledList}
            {victoryList.length > 0 && <Separator counter={victoryList.length}>victory</Separator>}
            {victoryList}
            {defeatList.length > 0 && <Separator counter={defeatList.length}>defeat</Separator>}
            {defeatList}
            {drawList.length > 0 && <Separator counter={drawList.length}>draw</Separator>}
            {drawList}
        </div>
    )
}

function Selectable({ game, selected, onClick }) {
    const myColor = game.myColor;
    const opponentColor = Color.opposite(myColor);
    const opponentName = game.opponentName;
    const flipBoard = (game?.myColor === Color.black) ? true : null;

    return (
        <div className={'Selectable' + (selected ? ' selected' : '')}
            onClick={() => onClick(game.uuid)}
        >
            <Board
                board={game.board}
                flip={flipBoard}
                prevMove={game.previousMove}
            />
            <div className='Message'>
                <Player
                    color={opponentColor}
                    name={opponentName}
                />
                <q>{game.message}</q>
            </div>
        </div>
    )
};

function Separator({ counter, children }) {
    return (
        <div className='Separator'>
            {children}
            <Counter number={counter} />
        </div>
    )
}